################################################################################
# IBM Confidential
# OCO Source Materials
# 5737-H76, 5725-W78, 5900-A1R
# (c) Copyright IBM Corp. 2020, 2022. All Rights Reserved.
# The source code for this program is not published or otherwise divested of its trade secrets,
# irrespective of what has been deposited with the U.S. Copyright Office.
################################################################################

import random
from .functions import (
    quantile_25,
    quantile_75,
    quantile_range,
    rate_of_change,
    sum_of_change,
    absoluate_sum_of_changes,
    trend_slop,
    abs_energy,
    mean_abs_change,
    mean_change,
    mean_second_derivative_central,
    count_above_mean,
    count_below_mean,
    corr_coefficient,
    delta_diff,
    past_value,
    fft_coefficient_real,
    fft_coefficient_abs,
    identity,
    wavelet_features,
    z_score,
    count_nan,
    geometric_mean,
    mode_abs_change,
    sum_log_values,
    normalised_count_below_mean,
    normalised_count_above_mean,
    discrete_cosine,
    svd_entropy,
    welch_spectral_entropy,
    fft_spectral_entropy,
    perm_entropy,
    katz_fd,
    petrosian_fd,
    max_langevin_fixed_point,
    fft_aggregated_kurtosis,
    fft_aggregated_skew,
    fft_aggregated_variance,
    fft_aggregated_centroid,
    partial_auto_correlation,
    augmented_dickey_fuller_used_lag,
    augmented_dickey_fuller_p_value,
    augmented_dickey_fuller_teststat,
    friedrich_coefficients,
    index_mass_quantile,
    energy_ratio_by_chunks,
    agg_auto_correlation_std,
    agg_auto_correlation_mean,
    agg_auto_correlation_median,
    agg_auto_correlation_var,
    symmetry_looking,
    #ar_coefficient,
    cid_ce,
    c3,
    time_reversal_asymmetry_statistic,
    auto_covariance,
    auto_correlation,
    approximate_entropy,
    has_duplicate,
    has_duplicate_min,
    has_duplicate_max,
    variance_larger_than_standard_deviation,
    mle_uniform,
    mle_gaussian,
    coefficient_variation,
    proportion_within_std_median,
    proportion_within_std_mean,
    unique_prop,
    trimmed_mean,
    mean_abs_dev,
    proportion_values_zero,
    proportion_values_positive,
    p_left,
    piecewise_aggregate_median,
    piecewise_aggregate_mean,
    std_outlier_test,
    mean_outlier_test,
    moments,
    histogram_mode,
    bowley_skewness,
    pearson_skewness,
    improved_burstiness,
    burstiness,
    high_low_mu,
    flat_spots,
    mean_crossing_points,
    median_crossing_points,
    stability,
    lumpiness,
    fisher,
    ans_combe,
    logit,
    relative_differencing,
    differencing,
    box_cox,
    quantize,
    cumsum,
    g_skew,
    anderson_darling,
    percent_amplitude,
    percent_difference_flux_percentile,
    flux_percentile_ratio_mid80,
    flux_percentile_ratio_mid65,
    flux_percentile_ratio_mid50,
    flux_percentile_ratio_mid35,
    flux_percentile_ratio_mid20,
    pair_slope_trend,
    median_buffer_range_percentage,
    median_abs_dev,
    small_kurtosis,
    outlier_segment_count,
    autocor_length,
    mean_variance,
    rate_of_cumulative_sum,
    amplitude,
    ratio_unique_value_number_to_time_series_length,
    sum_of_reoccurring_data_points,
    sum_of_reoccurring_values,
    percentage_of_reoccurring_values_to_all_values,
    percentage_of_reoccurring_datapoints_to_all_datapoints,
    longest_strike_above_mean,
    longest_strike_below_mean,
    large_standard_deviation,
    ratio_beyond_r_sigma,
    first_location_of_minimum,
    first_location_of_maximum,
    last_location_of_minimum,
    last_location_of_maximum,
    variance,
)


MAPPING = {
    "mean": "mean",
    "max": "max",
    "min": "min",
    "median": "median",
    "std": "std",
    "sum": "sum",
    "count": "count",
    "skew": "skew",
    "kurt": "kurt",
    "quantile_25": quantile_25,
    "quantile_75": quantile_75,
    "quantile_range": quantile_range,
    "rate_of_change": rate_of_change,
    "sum_of_change": sum_of_change,
    "absoluate_sum_of_change": absoluate_sum_of_changes,
    "trend_slop": trend_slop,
    "abs_energy": abs_energy,
    "mean_abs_change": mean_abs_change,
    "mean_change": mean_change,
    "mean_second_derivate_central": mean_second_derivative_central,
    "count_above_mean": count_above_mean,
    "count_below_mean": count_below_mean,
    "corr_coefficient": corr_coefficient,
    "delta_diff": delta_diff,
    "past_value": past_value,
    "fft_coefficient_real": fft_coefficient_real,
    "fft_coefficient_abs": fft_coefficient_abs,
    "fft_aggregated_centroid": fft_aggregated_centroid,
    "fft_aggregated_skew": fft_aggregated_skew,
    "variance": variance,
    "last_location_of_maximum": last_location_of_maximum,
    "last_location_of_minimum": last_location_of_minimum,
    "first_location_of_maximum": first_location_of_maximum,
    "first_location_of_minimum": first_location_of_minimum,
    "ratio_beyond_r_sigma": ratio_beyond_r_sigma,
    "large_standard_deviation": large_standard_deviation,
    "longest_strike_below_mean": longest_strike_below_mean,
    "longest_strike_above_mean": longest_strike_above_mean,
    "percentage_of_reoccurring_datapoints_to_all_datapoints": percentage_of_reoccurring_datapoints_to_all_datapoints,
    "percentage_of_reoccurring_values_to_all_values": percentage_of_reoccurring_values_to_all_values,
    "sum_of_reoccurring_values": sum_of_reoccurring_values,
    "sum_of_reoccurring_data_points": sum_of_reoccurring_data_points,
    "ratio_unique_value_number_to_time_series_length": ratio_unique_value_number_to_time_series_length,
    "amplitude": amplitude,
    "rate_of_cumulative_sum": rate_of_cumulative_sum,
    "mean_variance": mean_variance,
    "autocor_length": autocor_length,
    "outlier_segment_count": outlier_segment_count,
    "small_kurtosis": small_kurtosis,
    "median_abs_dev": median_abs_dev,
    "median_buffer_range_percentage": median_buffer_range_percentage,
    "pair_slope_trend": pair_slope_trend,
    "flux_percentile_ratio_mid20": flux_percentile_ratio_mid20,
    "flux_percentile_ratio_mid35": flux_percentile_ratio_mid35,
    "flux_percentile_ratio_mid50": flux_percentile_ratio_mid50,
    "flux_percentile_ratio_mid65": flux_percentile_ratio_mid65,
    "flux_percentile_ratio_mid80": flux_percentile_ratio_mid80,
    "percent_difference_flux_percentile": percent_difference_flux_percentile,
    "percent_amplitude": percent_amplitude,
    "anderson_darling": anderson_darling,
    "g_skew": g_skew,
    "cumsum": cumsum,
    "quantize": quantize,
    "box_cox": box_cox,
    "differencing": differencing,
    "relative_differencing": relative_differencing,
    "logit": logit,
    "ans_combe": ans_combe,
    "fisher": fisher,
    "lumpiness": lumpiness,
    "stability": stability,
    "median_crossing_points": median_crossing_points,
    "mean_crossing_points": mean_crossing_points,
    "flat_spots": flat_spots,
    "high_low_mu": high_low_mu,
    "burstiness": burstiness,
    "improved_burstiness": improved_burstiness,
    "pearson_skewness": pearson_skewness,
    "bowley_skewness": bowley_skewness,
    "histogram_mode": histogram_mode,
    "moments": moments,
    "mean_outlier_test": mean_outlier_test,
    "std_outlier_test": std_outlier_test,
    "piecewise_aggregate_mean": piecewise_aggregate_mean,
    "piecewise_aggregate_median": piecewise_aggregate_median,
    "p_left": p_left,
    "proportion_values_positive": proportion_values_positive,
    "proportion_values_zero": proportion_values_zero,
    "mean_abs_dev": mean_abs_dev,
    "trimmed_mean": trimmed_mean,
    "unique_prop": unique_prop,
    "proportion_within_std_mean": proportion_within_std_mean,
    "proportion_within_std_median": proportion_within_std_median,
    "coefficient_variation": coefficient_variation,
    "mle_gaussian": mle_gaussian,
    "mle_uniform": mle_uniform,
    "variance_larger_than_standard_deviation": variance_larger_than_standard_deviation,
    "has_duplicate_max": has_duplicate_max,
    "has_duplicate_min": has_duplicate_min,
    "has_duplicate": has_duplicate,
    "approximate_entropy": approximate_entropy,
    "auto_correlation": auto_correlation,
    "auto_covariance": auto_covariance,
    "time_reversal_asymmetry_statistic": time_reversal_asymmetry_statistic,
    "c3": c3,
    "cid_ce": cid_ce,
    #"ar_coefficient": ar_coefficient,
    "symmetry_looking": symmetry_looking,
    "agg_auto_correlation_var": agg_auto_correlation_var,
    "agg_auto_correlation_median": agg_auto_correlation_median,
    "agg_auto_correlation_mean": agg_auto_correlation_mean,
    "agg_auto_correlation_std": agg_auto_correlation_std,
    "energy_ratio_by_chunks": energy_ratio_by_chunks,
    "index_mass_quantile": index_mass_quantile,
    "friedrich_coefficients": friedrich_coefficients,
    "augmented_dickey_fuller_teststat": augmented_dickey_fuller_teststat,
    "augmented_dickey_fuller_p_value": augmented_dickey_fuller_p_value,
    "augmented_dickey_fuller_used_lag": augmented_dickey_fuller_used_lag,
    "partial_auto_correlation": partial_auto_correlation,
    "fft_aggregated_variance": fft_aggregated_variance,
    "fft_aggregated_kurtosis": fft_aggregated_kurtosis,
    "max_langevin_fixed_point": max_langevin_fixed_point,
    "petrosian_fd": petrosian_fd,
    "katz_fd": katz_fd,
    "perm_entropy": perm_entropy,
    "fft_spectral_entropy": fft_spectral_entropy,
    "welch_spectral_entropy": welch_spectral_entropy,
    "svd_entropy": svd_entropy,
    "discrete_cosine": discrete_cosine,
    "normalised_count_above_mean": normalised_count_above_mean,
    "normalised_count_below_mean": normalised_count_below_mean,
    "sum_log_values": sum_log_values,
    "mode_abs_change": mode_abs_change,
    "geometric_mean": geometric_mean,
    "count_nan": count_nan,
    "z_score": z_score,
    "wavelet_feature_haar": wavelet_features,
    "identity": identity,
}

def mapper(name):
    """
    Returns the transformation function based on the name provided by user. \
    Used in rolling window function.
    Parameters:
        name(string, required): Name of the function to return.
    Return:
        Aggregation of function name.
    """
    global MAPPING
    return MAPPING[name]


def get_random_featureset(feature_set_size):
    """
    Returns the transformation function based on the name provided by user. \
    Used in rolling window function.
    Parameters:
        name(string, required): Name of the function to return.
    Return:
        Aggregation of function name.
    """
    global MAPPING
    sample_features = random.sample(list(MAPPING.keys()), feature_set_size)
    pd_features = []
    np_features = []
    for item in sample_features:
        if item in [
            "mean",
            "max",
            "min",
            "median",
            "std",
            "sum",
            "count",
            "skew",
            "kurt",
        ]:
            pd_features.append(item)
        else:
            np_features.append(item)
    if len(pd_features) == 0:
        pd_features = None
    if len(np_features) == 0:
        np_features = None
    return pd_features, np_features
